@@ -307,11 +307,12 @@ int is_ko_point(int pos);
 int does_capture_something(int pos, int color);
 int is_self_atari(int pos, int color);
 
-/* Purely gemoetric functions */
+/* Purely geometric functions. */
 int is_edge_vertex(int pos);
 int is_corner_vertex(int pos);
 int edge_distance(int pos);
 int square_dist(int pos1, int pos2);
+int rotate1(int pos, int rot);
 
 /* Basic string information. */
 int find_origin(int str);
@@ -344,6 +345,9 @@ int countstones(int str);
 int findstones(int str, int maxstones, int *stones);
 int count_adjacent_stones(int str1, int str2, int maxstones);
 
+/* Detect a special shape. */
+int send_two_return_one(int move, int color);
+
 /* Special function for reading.c */
 void incremental_order_moves(int move, int color, int string,
 			     int *number_edges, int *number_same_string,
