@@ -2,12 +2,13 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
- * published by the Free Software Foundation - version 2             *
+ * published by the Free Software Foundation - version 3 or          *
+ * (at your option) any later version.                               *
  *                                                                   *
  * This program is distributed in the hope that it will be useful,   *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
@@ -408,14 +409,14 @@ atari_atari_blunder_size(int color, int move,
   memset(forbidden, 0, sizeof(forbidden));
   memset(defense_points, 0, sizeof(defense_points));
 
+  compute_aa_status(other, safe_stones);
+  compute_aa_values(other);
+
   /* Accept illegal ko capture here. */
   if (!tryko(move, color, NULL))
     /* Really shouldn't happen. */
     abortgo(__FILE__, __LINE__, "trymove", move);
 
-  compute_aa_status(other, safe_stones);
-  compute_aa_values(other);
-
   increase_depth_values();
 
   aa_val = do_atari_atari(other, &apos, &defense_point, defense_points,
@@ -518,6 +519,8 @@ atari_atari_blunder_size(int color, int move,
 
 /* Helper function for computing the aa_status for all opponent's strings.
  * If safe_stones is given, we just copy the information from there.
+ * If called at stackp > 0, safe_stones must be provided since the
+ * dragon_data is not valid then.
  */
 
 static void
@@ -528,6 +531,9 @@ compute_aa_status(int color, const signed char safe_stones[BOARDMAX])
   SGFTree *save_sgf_dumptree = sgf_dumptree;
   int save_count_variations = count_variations;
   int save_verbose = verbose;
+
+  gg_assert(safe_stones || stackp == 0);
+  
   sgf_dumptree = NULL;
   count_variations = 0;
   if (verbose)
@@ -572,8 +578,8 @@ compute_aa_status(int color, const signed char safe_stones[BOARDMAX])
    */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (board[pos] == other
-	&& worm[pos].origin == pos
-	&& worm[pos].liberties == 2
+	&& find_origin(pos) == pos
+	&& countlib(pos) == 2
 	&& aa_status[pos] == ALIVE) {
       int libs[2];
       findlib(pos, 2, libs);
@@ -587,7 +593,7 @@ compute_aa_status(int color, const signed char safe_stones[BOARDMAX])
       if (!owl_substantial(pos)) {
 	int pos2;
 	for (pos2 = BOARDMIN; pos2 < BOARDMAX; pos2++)
-	  if (ON_BOARD(pos2) && is_worm_origin(pos2, pos))
+	  if (board[pos2] == other && find_origin(pos2) == pos)
 	    aa_status[pos2] = INSUBSTANTIAL;
       }
     }
