@@ -52,6 +52,7 @@ init_gnugo(float memory, unsigned int seed)
 
   transformation_init();
   dfa_match_init();
+  choose_mc_patterns(NULL);
 
   clear_approxlib_cache();
   clear_accuratelib_cache();
@@ -60,6 +61,35 @@ init_gnugo(float memory, unsigned int seed)
 
 /* ---------------------------------------------------------------- */
 
+/* Check whether we can accept a certain boardsize. Set out to NULL to
+ * suppress informative messages. Return 1 for an acceptable
+ * boardsize, 0 otherwise.
+ */
+int check_boardsize(int boardsize, FILE *out)
+{
+  int max_board = MAX_BOARD;
+  if (use_monte_carlo_genmove && max_board > 9)
+    max_board = 9;
+  
+  if (boardsize < MIN_BOARD || boardsize > max_board) {
+    if (out) {
+      fprintf(out, "Unsupported board size: %d. ", boardsize);
+      if (boardsize < MIN_BOARD)
+	fprintf(out, "Min size is %d.\n", MIN_BOARD);
+      else {
+	fprintf(out, "Max size is %d", max_board);
+	if (max_board < MAX_BOARD)
+	  fprintf(out, " (%d without --monte-carlo)", MAX_BOARD);
+	fprintf(out, ".\n");
+      }
+      fprintf(out, "Try `gnugo --help' for more information.\n");
+    }
+    return 0;
+  }
+
+  return 1;
+}
+
 /*
  * Clear the board.
  */
@@ -162,7 +192,7 @@ gnugo_estimate_score(float *upper, float *lower)
     *upper = white_score;
   if (lower != NULL)
     *lower = black_score;
-  return ((white_score + black_score)/2.0);
+  return ((white_score + black_score) / 2.0);
 }
 
 
@@ -212,15 +242,6 @@ gameinfo_print(Gameinfo *gameinfo)
     printf("Nobody\n");
 }
 
-/* 1D interface to rotate(). */
-static int
-rotate1(int pos, int orientation)
-{
-  int ri, rj;
-  rotate(I(pos), J(pos), &ri, &rj, board_size, orientation);
-  return POS(ri, rj);
-}
-
 /*
  * Play the moves in an SGF tree. Walk the main variation, actioning
  * the properties into the playing board.
@@ -245,15 +266,9 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
   
   if (!sgfGetIntProperty(tree->root, "SZ", &bs))
     bs = 19;
-  
-  if (bs < MIN_BOARD || bs > MAX_BOARD) {
-    if (bs < MIN_BOARD)
-      gprintf("Boardsize too small.\n");
-    else
-      gprintf("Boardsize too large.\n");
-    
+
+  if (!check_boardsize(bs, stderr))
     return EMPTY;
-  }
   
   handicap = 0;
   if (sgfGetIntProperty(tree->root, "HA", &handicap) && handicap > 1)
