@@ -291,6 +291,7 @@ int confirm_safety(int move, int color, int *defense_point,
 int dragon_weak(int pos);
 float dragon_weakness(int pos, int ignore_dead_dragons);
 int size_of_biggest_critical_dragon(void);
+void change_dragon_status(int dr, enum dragon_status status);
 float blunder_size(int move, int color, int *defense_point,
 		   signed char safe_stones[BOARDMAX]);
 void set_depth_values(int level, int report_levels);
@@ -496,6 +497,16 @@ int aftermath_genmove(int color, int do_capture_dead_stones,
 		      int allowed_moves[BOARDMAX]);
 enum dragon_status aftermath_final_status(int color, int pos);
 
+int mc_get_size_of_pattern_values_table(void);
+int mc_load_patterns_from_db(const char *filename, unsigned int *values);
+void mc_init_patterns(const unsigned int *values);
+int choose_mc_patterns(char *name);
+void list_mc_patterns(void);
+
+void uct_genmove(int color, int *move, int *forbidden_moves,
+		 int *allowed_moves, int nodes, float *move_values,
+		 int *move_frequencies);
+
 int owl_attack(int target, int *attack_point, int *certain, int *kworm);
 int owl_defend(int target, int *defense_point, int *certain, int *kworm);
 int owl_threaten_attack(int target, int *attack1, int *attack2);
@@ -545,7 +556,6 @@ int does_attack(int move, int str);
 int does_defend(int move, int str);
 int double_atari(int move, int color, float *value,
 		 signed char safe_stones[BOARDMAX]);
-int send_two_return_one(int move, int color);
 int playing_into_snapback(int move, int color);
 int play_attack_defend_n(int color, int do_attack, int num_moves, ...);
 int play_attack_defend2_n(int color, int do_attack, int num_moves, ...);
@@ -894,10 +904,10 @@ struct dragon_data2 {
   int moyo_size;		      /* size of surrounding influence moyo, */
   float moyo_territorial_value;       /* ...and its territorial value        */
   enum dragon_status safety;          /* a more detailed status estimate     */
-  float weakness; /* A new (3.4) continuous estimate of the dragon's safety  */
-  float weakness_pre_owl;     /* Dragon safety based on pre-owl computations */
+  float weakness;           /* a continuous estimate of the dragon's safety  */
+  float weakness_pre_owl;   /* dragon safety based on pre-owl computations   */
   float strategic_size; /* An effective size including weakness of neighbors */
-  int escape_route; /* a measurement of likelihood of escape                 */
+  int escape_route;         /* a measurement of likelihood of escape         */
   struct eyevalue genus;    /* the number of eyes (approximately)            */
   int heye;     /* coordinates of a half eye                                 */
   int lunch;    /* if lunch != 0 then lunch points to a boundary worm which  */
