@@ -2,12 +2,13 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
- * published by the Free Software Foundation - version 2             *
+ * published by the Free Software Foundation - version 3 or          *
+ * (at your option) any later version.                               *
  *                                                                   *
  * This program is distributed in the hope that it will be useful,   *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
@@ -440,7 +441,7 @@ search_persistent_cache(struct persistent_cache *cache,
     print_persistent_cache_entry(entry);
   }
   /* FIXME: This is an ugly hack. */
-  if (cache->name == "reading cache"
+  if (strcmp(cache->name, "reading cache") == 0
       && (debug & DEBUG_READING_PERFORMANCE)
       && entry->cost >= MIN_READING_NODES_TO_REPORT) {
     if (entry->result != 0)
@@ -1313,7 +1314,7 @@ compute_active_semeai_area(struct persistent_cache_entry *entry,
   }
 
   compute_active_owl_type_area(goal, BLACK, active_b);
-  compute_active_owl_type_area(goal, WHITE, active_b);
+  compute_active_owl_type_area(goal, WHITE, active_w);
 
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     int value = board[pos];
