@@ -2,12 +2,13 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
- * published by the Free Software Foundation - version 2             *
+ * published by the Free Software Foundation - version 3 or          *
+ * (at your option) any later version.                               *
  *                                                                   *
  * This program is distributed in the hope that it will be useful,   *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
@@ -451,16 +452,14 @@ compute_surroundings(int pos, int apos, int showboard, int *surround_size)
   }
 
   if (surrounded)
-    for (m = 0; m < board_size; m++)
-      for (n = 0; n < board_size; n++) {
-        if (mf[POS(m, n)]) {
-	  if (mn[POS(m, n)] == 0) {
-	    surrounded = 0;
-	    break;
-	  }
-	  else if (mn[POS(m, n)] == 2)
-	    surrounded = WEAKLY_SURROUNDED;
-        }
+    for (dpos = BOARDMIN; dpos < BOARDMAX; dpos++)
+      if (mf[dpos]) {
+	if (mn[dpos] == 0) {
+	  surrounded = 0;
+	  break;
+	}
+	else if (mn[dpos] == 2)
+	  surrounded = WEAKLY_SURROUNDED;
       }
 
   /* revise the status for single stone dragons. */
