@@ -33,14 +33,14 @@
 #include "sgftree.h"
 #include "random.h"
 #include "gg_utils.h"
-
+#include "patterns.h"
 
 /*
  * Change the status of all the stones in the dragon at (dr).
  */
 
 void
-change_dragon_status(int dr, int status)
+change_dragon_status(int dr, enum dragon_status status)
 {
   int pos;
   int origin = dragon[dr].origin;
@@ -1407,63 +1407,6 @@ double_atari(int move, int color, float *value,
 }
     
 
-/* Returns true if a move by (color) fits a shape like:
- *
- *  -----
- *  O.O*X        (O=color)
- *  OOXXX
- *
- * More specifically the move should have the following properties:
- * - The move is a self-atari
- * - The move forms a string of exactly two stones
- * - When the opponent captures, the capturing stone becomes a single
- *   stone in atari
- * - When capturing back the original position is repeated
- */
-
-int
-send_two_return_one(int move, int color)
-{
-  int other = OTHER_COLOR(color);
-  int lib1;
-  int lib2;
-
-  /* Try to play the move. */
-  if (!trymove(move, color, "send_two_return_one-A", NO_MOVE))
-    return 0;
-
-  /* Does the move leave exactly two stones in atari? */
-  if (findlib(move, 1, &lib1) > 1 || countstones(move) != 2) {
-    popgo();
-    return 0;
-  }
-
-  /* Let the opponent capture. This move is guaranteed to be legal. */
-  trymove(lib1, other, "send_two_return_one-A", NO_MOVE);
-
-  /* Is the capturing string single and in atari? */
-  if (findlib(lib1, 1, &lib2) > 1 || countstones(lib1) > 1) {
-    popgo();
-    popgo();
-    return 0;
-  }
-
-  popgo();
-  popgo();
-
-  /* The move to capture back is in lib2. This must be one of the two
-   * captured stones, either the move or the adjacent stone. If it is
-   * the same as the first move, the previous position is not repeated
-   * by capturing back, so we return false.
-   */
-  if (lib2 == move)
-    return 0;
-
-  /* The move matches the shape. */
-  return 1;
-}
-
-
 /* Returns true if a move by (color) plays into a snapback. */
 int
 playing_into_snapback(int move, int color)
@@ -2016,6 +1959,43 @@ showstats()
 }
 
 
+/* Set up a compiled in pattern database for use by the Monte Carlo
+ * code. If name is NULL, the first pattern database is used.
+ *
+ * The reason why this function and the next are placed here rather
+ * than in montecarlo.c is to keep that file free from dependency on
+ * patterns.h.
+ */
+int
+choose_mc_patterns(char *name)
+{
+  int k;
+  for (k = 0; mc_pattern_databases[k].name; k++) {
+    if (!name || strcmp(name, mc_pattern_databases[k].name) == 0) {
+      mc_init_patterns(mc_pattern_databases[k].values);
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
+/* List compiled in Monte Carlo pattern databases. */
+void
+list_mc_patterns(void)
+{
+  int k;
+  printf("Available builtin Monte Carlo local patterns:\n\n");
+  for (k = 0; mc_pattern_databases[k].name; k++) {
+    if (k == 0)
+      printf("* %s (default)\n", mc_pattern_databases[k].name);
+    else
+      printf("* %s\n", mc_pattern_databases[k].name);
+  }
+  printf("\nUse \"--mc-patterns name\" to choose one of these.\n");
+  printf("Use \"--mc-load-patterns filename\" to directly load a pattern database.\n");
+}
+
 /*
  * Local Variables:
  * tab-width: 8
