@@ -157,7 +157,12 @@ enum {OPT_BOARDSIZE = 127,
       OPT_MIRROR_LIMIT,
       OPT_METAMACHINE,
       OPT_RESIGN_ALLOWED,
-      OPT_NEVER_RESIGN
+      OPT_NEVER_RESIGN,
+      OPT_MONTE_CARLO,
+      OPT_MC_GAMES_PER_LEVEL,
+      OPT_MC_PATTERNS,
+      OPT_MC_LIST_PATTERNS,
+      OPT_MC_LOAD_PATTERNS
 };
 
 /* names of playing modes */
@@ -229,7 +234,7 @@ static struct gg_option const long_options[] =
   {"backfill-depth", required_argument, 0, 'B'},
   {"branch-depth",   required_argument, 0, OPT_BRANCH_DEPTH},
   {"backfill2-depth",   required_argument, 0, OPT_BACKFILL2_DEPTH},
-  {"break_chain-depth", required_argument, 0, OPT_BREAK_CHAIN_DEPTH},
+  {"break-chain-depth", required_argument, 0, OPT_BREAK_CHAIN_DEPTH},
   {"superstring-depth", required_argument, 0, OPT_SUPERSTRING_DEPTH},
   {"fourlib-depth",  required_argument, 0, 'F'},
   {"ko-depth",       required_argument, 0, 'K'},
@@ -305,6 +310,11 @@ static struct gg_option const long_options[] =
   {"metamachine",    no_argument,       0, OPT_METAMACHINE},
   {"resign-allowed", no_argument,       0, OPT_RESIGN_ALLOWED},
   {"never-resign",   no_argument,       0, OPT_NEVER_RESIGN},
+  {"monte-carlo",    no_argument,       0, OPT_MONTE_CARLO},
+  {"mc-games-per-level", required_argument, 0, OPT_MC_GAMES_PER_LEVEL},
+  {"mc-patterns",    required_argument, 0, OPT_MC_PATTERNS},
+  {"mc-list-patterns", no_argument,     0, OPT_MC_LIST_PATTERNS},
+  {"mc-load-patterns", required_argument, 0, OPT_MC_LOAD_PATTERNS},
   {NULL, 0, NULL, 0}
 };
 
@@ -340,6 +350,9 @@ main(int argc, char *argv[])
   FILE *output_check;
   int orientation = 0;
 
+  char mc_pattern_name[40] = "";
+  char mc_pattern_filename[320] = "";
+
   float memory = (float) DEFAULT_MEMORY; /* Megabytes used for hash table. */
 
   /* If seed is zero, GNU Go will play a different game each time. If
@@ -348,6 +361,8 @@ main(int argc, char *argv[])
    */
   int seed = 0;
   int seed_specified = 0;
+  
+  int requested_boardsize = -1;
 
   sgftree_clear(&sgftree);
   gameinfo_clear(&gameinfo);
@@ -385,6 +400,10 @@ main(int argc, char *argv[])
 	break;
 	
       case 'o':
+	if (strlen(gg_optarg) >= sizeof(outfilename)) {
+	  fprintf(stderr, "Too long filename given as value to -o option.\n");
+	  exit(EXIT_FAILURE);
+	}
 	outfile = gg_optarg;
 	strcpy(outfilename, gg_optarg);
 	break;
@@ -469,7 +488,7 @@ main(int argc, char *argv[])
 		  "configure option enabled: owl threats\n");
 	if (RESIGNATION_ALLOWED)
 	  fprintf(stdout,
-		  "configure option enabled: resination allowed\n");
+		  "configure option enabled: resignation allowed\n");
 	if (ORACLE)
 	  fprintf(stdout,
 		  "configure option enabled: oracle\n");
@@ -508,21 +527,8 @@ main(int argc, char *argv[])
         break;
       
       case OPT_BOARDSIZE:
-        {
-	  int boardsize = atoi(gg_optarg);
-	  
-	  if (boardsize < MIN_BOARD || boardsize > MAX_BOARD) {
-	    fprintf(stderr, "Unsupported board size: %d. ", boardsize);
-	    if (boardsize < MIN_BOARD)
-	      fprintf(stderr, "Min size is %d.\n", MIN_BOARD);
-	    else
-	      fprintf(stderr, "Max size is %d.\n", MAX_BOARD);
-	    fprintf(stderr, "Try `gnugo --help' for more information.\n");
-	    exit(EXIT_FAILURE);
-	  }
-	  gnugo_clear_board(boardsize);
-	  break;
-	}
+	requested_boardsize = atoi(gg_optarg);
+	break;
 	
       case OPT_KOMI: 
 	if (sscanf(gg_optarg, "%f", &komi) != 1) {
@@ -640,6 +646,35 @@ main(int argc, char *argv[])
 	resign_allowed = 0;
 	break;
 
+      case OPT_MONTE_CARLO:
+	use_monte_carlo_genmove = 1;
+	break;
+
+      case OPT_MC_GAMES_PER_LEVEL:
+	mc_games_per_level = atoi(gg_optarg);
+	break;
+
+      case OPT_MC_PATTERNS:
+	if (strlen(gg_optarg) >= sizeof(mc_pattern_name)) {
+	  fprintf(stderr, "Too long name given as value to --mc-patterns option.\n");
+	  exit(EXIT_FAILURE);
+	}
+	strcpy(mc_pattern_name, gg_optarg);
+	break;
+
+      case OPT_MC_LIST_PATTERNS:
+	list_mc_patterns();
+	return EXIT_SUCCESS;
+	break;
+
+      case OPT_MC_LOAD_PATTERNS:
+	if (strlen(gg_optarg) >= sizeof(mc_pattern_filename)) {
+	  fprintf(stderr, "Too long name given as value to --mc-load-patterns option.\n");
+	  exit(EXIT_FAILURE);
+	}
+	strcpy(mc_pattern_filename, gg_optarg);
+	break;
+
       case OPT_MODE: 
 	if (strcmp(gg_optarg, "ascii") == 0)
 	  playmode = MODE_ASCII;
@@ -969,6 +1004,12 @@ main(int argc, char *argv[])
       }
     }
 
+  if (requested_boardsize != -1) {
+    if (!check_boardsize(requested_boardsize, stderr))
+      exit(EXIT_FAILURE);
+    gnugo_clear_board(requested_boardsize);
+  }
+  
   /* Start random number seed. */
   if (!seed_specified)
     seed = time(0);
@@ -976,6 +1017,23 @@ main(int argc, char *argv[])
   /* Initialize the GNU Go engine. */
   init_gnugo(memory, seed);
 
+  /* Load Monte Carlo patterns if one has been specified. Either
+   * choose one of the compiled in ones or load directly from a
+   * database file.
+   */
+  if (strlen(mc_pattern_filename) > 0) {
+    if (!mc_load_patterns_from_db(mc_pattern_filename, NULL))
+      return EXIT_FAILURE;
+  }
+  else if (strlen(mc_pattern_name) > 0) {
+    if (!choose_mc_patterns(mc_pattern_name)) {
+      fprintf(stderr, "Unknown Monte Carlo pattern database name %s.\n",
+	      mc_pattern_name);
+      fprintf(stderr, "Use \"--mc-list-patterns\" to list the available databases.\n");
+      return EXIT_FAILURE;
+    }
+  }
+
   /* Read the infile if there is one. Also play up the position. */
   if (infilename) {
     if (!sgftree_readfile(&sgftree, infilename)) {
@@ -1523,6 +1581,11 @@ Experimental options:\n\
    --nojosekidb            turn off joseki database\n\
    --mirror                try to play mirror go\n\
    --mirror-limit <n>      stop mirroring when n stones on board\n\n\
+   --monte-carlo           enable Monte Carlo move generation (9x9 or smaller)\n\
+   --mc-games-per-level <n> number of Monte Carlo simulations per level\n\
+   --mc-list-patterns      list names of builtin Monte Carlo patterns\n\
+   --mc-patterns <name>    choose a built in Monte Carlo pattern database\n\
+   --mc-load-patterns <filename> read Monte Carlo patterns from file\n\
    --alternate-connections\n\
    --experimental-connections\n\
    --experimental-owl-ext\n\
@@ -1740,7 +1803,7 @@ socket_connect_to(const char *host_name, unsigned int port,
   }
 
   if (! *address_pointer) {
-    fprintf(stderr, "Failed to connect to %s:%d\n", host_data->h_name, port);
+    fprintf(stderr, "Failed to connect to %s:%u\n", host_data->h_name, port);
     closesocket(connection_socket);
     exit(EXIT_FAILURE);
   }
@@ -1800,11 +1863,11 @@ socket_listen_at(const char *host_name, unsigned int port,
 
   if (verbose) {
     if (host_name) {
-      fprintf(stderr, "Waiting for a connection on %s:%d...\n",
+      fprintf(stderr, "Waiting for a connection on %s:%u...\n",
 	      host_name, port);
     }
     else
-      fprintf(stderr, "Waiting for a connection on port %d...\n", port);
+      fprintf(stderr, "Waiting for a connection on port %u...\n", port);
   }
 
   if (bind(listening_socket,
@@ -1812,9 +1875,9 @@ socket_listen_at(const char *host_name, unsigned int port,
       || listen(listening_socket, 0) == -1
       || (connection_socket = accept(listening_socket, NULL, NULL)) == -1) {
     if (host_name)
-      fprintf(stderr, "Failed to listen on %s:%d\n", host_name, port);
+      fprintf(stderr, "Failed to listen on %s:%u\n", host_name, port);
     else
-      fprintf(stderr, "Failed to listen on port %d\n", port);
+      fprintf(stderr, "Failed to listen on port %u\n", port);
 
     closesocket(listening_socket);
     exit(EXIT_FAILURE);
