@@ -2,12 +2,13 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
- * published by the Free Software Foundation - version 2             *
+ * published by the Free Software Foundation - version 3 or          *
+ * (at your option) any later version.                               *
  *                                                                   *
  * This program is distributed in the hope that it will be useful,   *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of    *
@@ -436,6 +437,7 @@ static struct autohelper_func autohelper_functions[] = {
   {"adjacent_to_defendable_stone_in_atari", 1, 0, 1.0,
                 "adjacent_to_defendable_stone_in_atari(%s)"},
   {"good_attack_threat",	2, 0, 0.01, "register_good_attack_threat(%s, %s)"},
+  {"known_safe_move",		1, 0, 0.01, "register_known_safe_move(%s)"},
   {"break_mirror_helper",	1, 0, 0.01, "break_mirror_helper(%s, color)"}
 };
 
@@ -455,12 +457,13 @@ dummyhelper(int transformation, int move, int color, int action)
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *\n\
  * http://www.gnu.org/software/gnugo/ for more information.          *\n\
  *                                                                   *\n\
- * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005 and 2006       *\n\
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 and 2007 *\n\
  * by the Free Software Foundation.                                  *\n\
  *                                                                   *\n\
  * This program is free software; you can redistribute it and/or     *\n\
  * modify it under the terms of the GNU General Public License as    *\n\
- * published by the Free Software Foundation - version 2             *\n\
+ * published by the Free Software Foundation - version 3 or          *\n\
+ * (at your option) any later version.                               *\n\
  *                                                                   *\n\
  * This program is distributed in the hope that it will be useful,   *\n\
  * but WITHOUT ANY WARRANTY; without even the implied warranty of    *\n\
@@ -915,7 +918,8 @@ read_pattern_line(char *p)
     }
     
     /* Range checking. */
-    assert(el < (int) (sizeof(elements) / sizeof(elements[0])));
+    if (el >= (int) (sizeof(elements) / sizeof(elements[0])))
+      return 0;
     
     elements[el].x = maxi;
     elements[el].y = j;
@@ -1007,7 +1011,7 @@ read_constraint_diagram_line(char *p)
    * maxj was modified by find_extents() so we have to compensate for
    * this.
    */
-  if (j != maxj + 1) {
+  if (j != maxj + 1 && !discard_pattern) {
     fprintf(stderr, "%s(%d) : error : Mismatching width of constraint line in pattern %s\n", 
 	    current_file, current_line_number, pattern_names[patno]);
     fatal_errors++;
@@ -1031,7 +1035,7 @@ read_constraint_diagram_line(char *p)
 static void
 check_constraint_diagram_size(void)
 {
-  if (current_c_i != maxi + 1) {
+  if (current_c_i != maxi + 1 && !discard_pattern) {
     fprintf(stderr, "%s(%d) : error : Mismatching height of constraint diagram in pattern %s\n", 
 	    current_file, current_line_number, pattern_names[patno]);
     fatal_errors++;
@@ -1096,7 +1100,7 @@ finish_pattern(char *line)
     ci = 0;
     cj = 0;
   }
-  else if (choose_best_anchor) { 
+  else if (choose_best_anchor && !discard_pattern) { 
 
     /* Try to find a better anchor if
      * the -m option is set.
@@ -1125,7 +1129,7 @@ finish_pattern(char *line)
     pattern[patno].anchored_at_X = (elements[min_k].att == ATT_X) ? 3 : 0;
     
   }
-  else if (ci == -1 || cj == -1) {
+  else if ((ci == -1 || cj == -1) && !discard_pattern) {
     fprintf(stderr, "%s(%d) : No origin for pattern %s\n", 
 	    current_file, current_line_number, pattern_names[patno]);
     fatal_errors = 1;
