@@ -293,6 +293,11 @@ struct fullboard_pattern {
 };
 
 
+/* Monte Carlo local patterns. */
+struct mc_pattern_database {
+  const char *name;
+  const unsigned int *values;
+};
 
 
 /* helper functions */
@@ -326,6 +331,7 @@ int adjacent_to_stone_in_atari(int str);
 int adjacent_to_defendable_stone_in_atari(int str);
 void backfill_replace(int move, int str);
 int break_mirror_helper(int str, int color);
+int distrust_tactics_helper(int str);
 int disconnect_helper(int apos, int bpos);
 
 
@@ -351,6 +357,8 @@ extern struct fullboard_pattern fuseki19[];
 extern struct fullboard_pattern fuseki13[];
 extern struct fullboard_pattern fuseki9[];
 
+extern struct mc_pattern_database mc_pattern_databases[];
+
 struct corner_db;
 struct corner_variation;
 struct corner_pattern;
